
/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 * RTRT  Tracer, Performance profiling, and Coverage example
 *
 * description : Philosopher dinning
 * 		need to have right and left fork available at the same time to eat.
 *		 but left and right forks are shared with the next philosophers
 *
 */
#include "Philosopher.h"

#define NB_THREAD 5
struct Mutex go;

void * run( struct Philosopher * const this )
{
	Mutex_take(&go,INFINITE);
	Mutex_release(&go);
	return Philosopher_run (this);
}

int main( int argc, char ** argv )
{
	struct Philosopher philo[NB_THREAD];
	struct Mutex mut[NB_THREAD];
	int threadNo;
	HANDLE thread[NB_THREAD];
	long resultThread;

	philo_init();
	fork_init();
	Mutex_init(&go);
	Mutex_take(&go,INFINITE);

	for(threadNo=0; threadNo<NB_THREAD; threadNo++)
	{
		Mutex_init(&mut[threadNo]);
		mut[threadNo].id = threadNo + 1;
		philo[threadNo].rightFork = &mut[threadNo];
		philo[threadNo].leftFork = &mut[(threadNo + 1) % NB_THREAD];
		philo[threadNo].id = threadNo + 1;

		thread[threadNo] = CreateThread( 0, 0,
			(LPTHREAD_START_ROUTINE)Philosopher_run,
			(LPVOID)&philo[threadNo], 0, 0 );
	}

#ifdef ATTOLDUMP
	_atl_obstools_dump(1);
#endif

	Mutex_release(&go);

	threadNo=0;
	while(threadNo<NB_THREAD)
	{	 		//_cwait(&resultThread, HndThread[threadNo], NULL);
		GetExitCodeThread(thread[threadNo], &resultThread);
		if(resultThread == STILL_ACTIVE)
		{
			Sleep(1000);
#pragma att_insert _ATT_FLUSH_ITEMS
		}
		else
		{
			printf("instance %d (No %d) exit code: %8X\n", threadNo, thread[threadNo],resultThread);
			threadNo++;
		}
	}

	exit(0);
	return 0;
}
