

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */



#include <stdio.h>
#include "GreedyPhilosopher.h"

// ========================================================================
// Constructor of a philosopher
GreedyPhilosopher::GreedyPhilosopher (MaitreDHote& theBoss)
  : Philosopher (theBoss)
{
}

// ========================================================================
// When the philosopher hasn't any forks it tries to get the one on the left.

void GreedyPhilosopher::doHasNoForks ()
{
  theLeftFork->GetFork (*this, INFINITE);
  setState (hasOneFork);
}

// ========================================================================
// When the philosopher has one fork and waits for the second to be available

void GreedyPhilosopher::doHasOneFork ()
{
  if (theRightFork->GetFork (*this, INFINITE))
    setState (hasTwoForks);
}

// ========================================================================
// When the philosopher has both forks it informs the maitre d'hote
// and starts eating

void GreedyPhilosopher::doHasTwoForks ()
{
  setState (isEating);
  theMaitreDHote.amEating(*this);
}

// ========================================================================
// The philosopher takes some time to eat then starts thinking

void GreedyPhilosopher::doIsEating ()
{
  // 0,02 seconds
  Sleep (20);
  setState (isThinking);
}

// ========================================================================
// Before thinking the philosopher releases the forks

void GreedyPhilosopher::doIsThinking ()
{
  theRightFork->Release();
  theLeftFork->Release();
  // Think for 0,02 seconds
  Sleep (20);
  numberOfTimesEaten++;
  if (hasEatenEnough())
    setState (finishedEating);
  else
    setState (hasNoForks);
  char note[256];
  sprintf (note, "Eaten %d times", numberOfTimesEaten);
#pragma attol att_insert _ATT_USER_NOTE (note)
}
