

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


#include <stdio.h>
#include "philosopher.h"
// #include <iostream.h>

// ========================================================================
// The thread main function
DWORD threadMain (LPDWORD lpdwParam)
{
  Philosopher* pInstance = (Philosopher*)lpdwParam;
  pInstance->run();
  return (0);
}

// ========================================================================
// Constructor of a philosopher
Philosopher::Philosopher (MaitreDHote& theBoss)
  : theMaitreDHote(theBoss)
{
  theCurrentState = hasNoForks;
  theLeftFork = 0;
  theRightFork = 0;
  numberOfTimesEaten = 0;

  pMyThread = Thread::getNewThread ((int (*)(void*))threadMain,
				    (void*)this);
}

// ========================================================================
// Constructor of a philosopher
Philosopher::~Philosopher ()
{
  theLeftFork = 0;
  theRightFork = 0;
  delete pMyThread;
}

// ========================================================================
// Sets a new state
void Philosopher::setState (States newState)
{
  theCurrentState = newState;
}

// ========================================================================
// Start the philosopher eating
void Philosopher::Dine ()
{
  pMyThread->resumeThread ();
}



// ========================================================================
// Returns true when the philosopher has eaten enough
bool Philosopher::hasEatenEnough () const
{
  // cout << "hasEatenEnough:numberOfTimesEaten = " << numberOfTimesEaten << endl;
  return (numberOfTimesEaten == 5);
}


// ========================================================================
// Changes the state of the philosopher, implements the philosophers
// dining strategy
void Philosopher::run()
{
  while (continueEating()) {
  }
}

// ========================================================================
// Changes the state of the philosopher, implements the philosophers
// dining strategy
bool Philosopher::continueEating()
{
  // cout << "continueEating:numberOfTimesEaten = " << numberOfTimesEaten
  //      << endl;
  // What's the current state
#pragma attol att_insert _ATT_USER_NOTE (getState())
  switch (theCurrentState) {
  case hasNoForks:
    this->doHasNoForks();
    break;
  case hasOneFork:
    this->doHasOneFork();
    break;
  case hasTwoForks:
    this->doHasTwoForks();
    break;
  case isEating:
    this->doIsEating();
    break;
  case isThinking:
    this->doIsThinking();
    break;
  case finishedEating:
    this->doFinishedEating();
    break;
  }
#pragma attol att_insert _ATT_USER_NOTE (getState())
  return ! hasEatenEnough();
}


// ========================================================================
// Changes the state of the philosopher, implements the philosophers
// dining strategy
const char* Philosopher::getState() const
{
  // cout << "continueEating:numberOfTimesEaten = " << numberOfTimesEaten
  //      << endl;
  // What's the current state
  switch (theCurrentState) {
  case hasNoForks:
    return "hasNoForks";
  case hasOneFork:
    return "hasOneFork";
  case hasTwoForks:
    return "hasTwoForks";
  case isEating:
    return "isEating";
  case isThinking:
    return "isThinking";
  case finishedEating:
    return "finishedEating";
  }
}


// ========================================================================
// When the philosopher hasn't any forks it tries to get the one on the left.

void Philosopher::doHasNoForks ()
{
  theLeftFork->GetFork (*this, INFINITE);
  setState (hasOneFork);
}

// ========================================================================
// When the philosopher has one fork it tries to get the one on the right for
// a period of 0,002 seconds. If it can't acquire the fork it will release all
// forks and sleep for 0,2 seconds.

void Philosopher::doHasOneFork ()
{
  if (theRightFork->GetFork (*this, 2))
    setState (hasTwoForks);
  else
    {
      // Didn't get the right fork so release the left fork
      theLeftFork->Release();
      Sleep (200);
      setState (hasNoForks);
    }
}

// ========================================================================
// When the philosopher has both forks it informs the maitre d'hote
// and starts eating

void Philosopher::doHasTwoForks ()
{
  setState (isEating);
  theMaitreDHote.amEating(*this);
}

// ========================================================================
// The philosopher takes some time to eat then starts thinking

void Philosopher::doIsEating ()
{
  // 0,2 seconds
  Sleep (200);
  setState (isThinking);
}

// ========================================================================
// Before thinking the philosopher releases the forks

void Philosopher::doIsThinking ()
{
  theRightFork->Release();
  theLeftFork->Release();
  // Think for 2 seconds
  Sleep (200);
  numberOfTimesEaten++;
  if (hasEatenEnough())
    setState (finishedEating);
  else
    setState (hasNoForks);
  char note[256];
  sprintf (note, "Eaten %d times", numberOfTimesEaten);
#pragma attol att_insert _ATT_USER_NOTE (note)
}

// ========================================================================
// The philosopher has finished eating so the tread terminates

void Philosopher::doFinishedEating ()
{
  // Do nothing as the state machine will automatically exit the thread
}

