

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


#ifndef PHILOSOPHER_H
#define PHILOSOPHER_H

#include <windows.h>
#include "fork.h"
#include "maitredhote.h"
#include "thread.h"

class Philosopher {
  
private:

  // The handle of the philosophers thread
  Thread* pMyThread;

protected:
  // The philosopher is implemented as a state machine with the following
  // states
  enum States { hasNoForks,
		hasOneFork,
		hasTwoForks,
		isEating,
		isThinking,
		finishedEating
  };

private:
  // Maintains the current state of the philosopher
  States theCurrentState;

protected:
  // References to the mutex locks (forks :-)
  Fork* theLeftFork;
  Fork* theRightFork;

  // Reference to matre d'hte to inform him when we are eating
  MaitreDHote& theMaitreDHote;

  int numberOfTimesEaten;

  // The functions that handle each state and change it. They are virtual so
  // they can be overwritten
  virtual void doHasNoForks ();
  virtual void doHasOneFork();
  virtual void doHasTwoForks();
  virtual void doIsEating();
  virtual void doIsThinking();
  virtual void doFinishedEating();

  // Sets a new state
  void setState (States newState);

public:

  // Constructor
  Philosopher (MaitreDHote& theBoss);

  // Desstructor
  ~Philosopher ();

  // Assign the left fork to the philosopher
  void assignLeftFork (Fork *pFork) {
    theLeftFork = pFork;
  }

  // Assign the right fork to the philosopher
  void assignRightFork (Fork *pFork) {
    theRightFork = pFork;
  }

  // Start the thread
  void Dine ();

  // Stop the thread and return the return code
  void StopEating ();

  // Get the current State as a string
  const char* getState () const;

  // Get the current thread handle
  Thread* getThread () const {
    return pMyThread;
  }

  // Returns true when the philosopher has eaten enough
  bool hasEatenEnough () const;

  // The main thread routine
  void run();

  // Changes the state of the philosopher, implements the philosophers
  // dining strategy. Returns true if the philosopher hasn't eaten enough
  virtual bool continueEating();

};

  
#endif
