

/* 
 *  Licensed Materials - Property of IBM
 *  Use Restricted Material of IBM - please refer to the "Samples Code" terms and conditions in the IBM International Program License Agreement (IPLA).
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


#ifndef MUTEX_H
#define MUTEX_H

class Mutex {
 protected:
  // You must use  the static function to create a new mutex object
  Mutex() {};
  Mutex(const Mutex& m);

 public:
  // Returns a new Mutex object. Throws an exception in case of error 
  // (MutexCreationFailed).
  static Mutex* getNewMutex();

  // Destroys the mutex
  virtual ~Mutex() = 0;

  // Returns true if the mutex was obtained with the given time. false
  // otherwise. Throws an exception in case of error (MutexLockFailed).
  virtual bool getMutex (int MaxTimeInMilliSeconds) = 0;

  // Releases the mutex. Throws an exception in case of error
  // (MutexReleaseFailed).
  virtual void releaseMutex() = 0;

  // Exception classes
  class MutexCreationFailed { };
  class MutexLockFailed { };
  class MutexReleaseFailed { };
};

#endif
